#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "ffft.h"

#define SET_ADC_CH(ch) {	\
		ADMUX&=0b11111000;	\
		ADMUX|=((ch)&7);	\
	}
	
#define LED1 0
#define LED2 1
#define LED3 2
#define LED4 3
#define LED5 4
#define LED6 5
#define LED7 6
#define LED8 7

/* initialize ADC */
void ADC_init(void){
	PORTA = 0;

	/* AVCC with external capacitor at AREF pin */
	ADMUX = (1<<REFS0);
	/* Set division factor to 128 and enable ADC (ADEN) */
	ADCSRA = (1<<ADEN) | (1<<ADPS1)| (1<<ADPS0) | (0<<ADPS2);

}

/*read data from ADC */
int16_t ADC_get(uint8_t chanel){
	ADMUX &= 0xE0;		                 //Save the first 3 bits and set the rest to 0
	ADMUX = chanel;						 //Set the MUXn bits
	ADCSRA |= (1<<ADSC);				 //Start ADC
	while(!(ADCSRA & (1<<ADIF)));		 //Loop until the data is ready
	ADCSRA |= (1<<ADIF);
	return ADC;
}

/* main -- Program starts here */
int main(void)
{	
	int i, value;
	uint8_t channel = PA0;				//the pin where the audio jack is connected
	int16_t buffer[FFT_N];				//buffer for captured values type as specified in ffft.h
	complex_t complex_buffer[FFT_N];    //buffer for the Fast Fourier Transformation 
	int16_t result[FFT_N];
	
	SET_ADC_CH(channel);
	DDRC = 0xFF;						// setam toti pinii portului C ca pini de iesire
	while(1)
	{
		/* Get FFT_N(64 - defined in ffft.h) values */
		for (i=0;i<FFT_N;i++)
			buffer[i] = ADC_get(channel);
		_delay_ms(20);
		
		fft_input(buffer, complex_buffer);
		fft_execute(complex_buffer);
		fft_output(complex_buffer, result);

		/*Light LEDS for spectrum */
		for (i=0; i<FFT_N; i++)
		{
			value = result[i] % 8;
			_delay_ms(2);
			switch(value){
				case LED1:
					PORTC = PORTC ^ 0x01;
					break;
				case LED2:
					PORTC = PORTC ^ 0x02;
					break;
				case LED3:
					PORTC = PORTC ^ 0x03;
					break;
				case LED4:
					PORTC = PORTC ^ 0x04;
					break;
				case LED5:
					PORTC = PORTC ^ 0x05;
					break;
				case LED6:
					PORTC = PORTC ^ 0x06;
					break;
				case LED7:
					PORTC = PORTC ^ 0x07;
					break;
				case 8:
					PORTC = PORTC ^ 0x08;
					break;
				default:
					return 0;
					break;
			}
		}
	}

	return 0;
}
